////////////////////////////////////////////////
//
//
//
////////////////////////////////////////////////

#ifndef _AI_INTERFACE_H
#define _AI_INTERFACE_H

#include <stdarg.h>		// For variable argument lengths
#include <stdio.h>		// for formating arguments like printf

#define		NUM_SENSORS			8
#define		SENSOR_MAX_RANGE	100.0f
#define		GRAVITY				1.63		// (Moon gravity constant = 1.63 m/s^2)




namespace 	Lunar_AI
{
	__declspec(dllexport) void Init( );

	__declspec(dllexport) void Cleanup( float a_final_score );

	__declspec(dllexport) void Tick( float a_delta_time,		// Time since last Tick() call. Always 0.01 (seconds)
					float a_lander_x, float a_lander_y,			// World-space position of your lunar lander (meters)
					float a_lander_x_vel, float a_lander_y_vel, // Current velocity of your lunar lander (meters per second)
					float a_lander_rotation,					// Rotation of your lunar lander (radians). 0.0 is upright.
					float a_goal_x, float a_goal_y,				// World-space position of the landing pad (meters)
					float a_sensor_data[ NUM_SENSORS ],			// Distance checks coming out of your lunar lander to warn of obstacles.
																	// [0] = up-left, [1] = up, [2] = up-right, [3] = left
																	// [4] = right, [5] = down-left, [6] = down, [7] = down-right
																	// If there is no obstacle, it will return SENSOR_MAX_RANGE ( meters)
					float a_fuel_left,							// How much fuel you have remaining (1.0 = full tank, 0.0 = empty).
					bool &a_out_thrusters_on,					// Set to true to turn on boosters.
					bool &a_out_rotate_left,					// Set to true to turn lunar lander left (counter clockwise).
					bool &a_out_rotate_right);					// Set to true to turn lunar lander right (clockwise).

	__declspec(dllexport) const char * Get_AI_Name( );


	/////////////////////////////////////////////////////
	// Debug support
	typedef void Draw_Text_Function( float, float, float, bool, unsigned __int32, const char * );
	typedef void Draw_Line_Function( float, float, float, float, bool, unsigned __int32 );
	static Draw_Text_Function *s_draw_text_function = 0;
	static Draw_Line_Function *s_draw_line_function = 0;

	__declspec(dllexport) void Set_Callback_Functions( Draw_Text_Function * a_draw_text_function, Draw_Line_Function * a_draw_line_function )
	{
		s_draw_text_function = a_draw_text_function;
		s_draw_line_function = a_draw_line_function;
	}

	void Draw_Text( float a_x, float a_y, float a_scale_multiplier, bool a_screen_coords, unsigned __int32 a_color, const char * a_format, ... )
	{
		#define		MAX_LOG_MESSAGE_LENGTH		512
		static char		log_msg[ MAX_LOG_MESSAGE_LENGTH ];

		if ( s_draw_text_function )
		{
			va_list arg_list;
			va_start( arg_list, a_format );
			::vsprintf_s( log_msg, MAX_LOG_MESSAGE_LENGTH, a_format, arg_list );
			va_end( arg_list );
			s_draw_text_function( a_x, a_y, a_scale_multiplier, a_screen_coords, a_color, log_msg );
		}
	}

	void Draw_Line( float a_x1, float a_y1, float a_x2, float a_y2, bool a_screen_coords, unsigned __int32 a_color )
	{
		if ( s_draw_line_function )
		{
			s_draw_line_function( a_x1, a_y1, a_x2, a_y2, a_screen_coords, a_color );
		}
	}
	////////////////////////////////////////////////////
};


#endif // _AI_INTERFACE_H













